#ifndef _FACE_TRACK_H
#define _FACE_TRACK_H

#define TWOPI_THREE_HALVES	15.749609946
#define EPSILON				1e-10
#define MIN_FLOAT			1e-20
#define LN_MIN_FLOAT		-46
#define MINFEATURESIZE		5
#define LARGENUM			1000000

#define PACKRGB(r,g,b) ((((unsigned int) r) & 0xfc) << 10) | ((((unsigned int) g) & 0xfc) << 4) | ((((unsigned int) b) & 0xfc) >> 2)
#define UNPACKR(x) (((unsigned int) x) & 0x3f000) >> 10
#define UNPACKG(x) (((unsigned int) x) & 0x00fc0) >> 4
#define UNPACKB(x) (((unsigned int) x) & 0x0003f) << 2

enum TEMPLATE_TYPE
{
	Ft_RECTANGLE,
	Ft_ELLIPSE,
	Ft_PARABOLA
};

typedef struct tagTEMPLATE
{
	int x1;			//left
	int y1;			//top
	int x2;			//right
	int y2;			//bottom
	int center;		//vertical center
}FACE_RECT;

typedef struct tagGAUSS
{
	double c;
	double c_acc;						//acumulate
	double mean[3], mean_acc[3];
	double cov[3][3], covinv[3][3];
	double cov_acc[3][3];
	double gconst;
}GAUSS;

typedef struct tagSTATE
{
	int nels;
	GAUSS mix[2];
}STATE;

//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
class AFX_EXT_CLASS CFaceTrack
{
public:
	double *FtGetTemplate() const
	{
		return (double*)m_fTemplate;
	}

public:
	//constructor
	CFaceTrack();
	//destructor
	virtual ~CFaceTrack();

public:
	//high level member functions
	BOOL FtInitialization (int ImgWidth, int ImgHeight, BOOL RestrictAspectRatio=TRUE,
							TEMPLATE_TYPE TemplateType=Ft_ELLIPSE);

	BOOL FtTraining (RECT FaceLocation, unsigned char *ImgData);

	BOOL FtTrackNextFrame (RECT &FaceRect, unsigned char *ImgData);

	//low level member functions
	//fill the mask with current template shape
	void FtFillMask (int ImgWidth, int ImgHeight, unsigned char *pMask, double *Template, 
						FACE_RECT FaceRect, int flag);

	//calculate the template shape parameters
	void FtFillTemplateParabola();
	void FtFillTemplateEllipse();
	void FtFillTemplateRectangle();

	//split the color channels
	void FtSplitColorChannel (int ImgWidth, int ImgHeight, unsigned char *pImgData, unsigned char *pRGBSplitImg[3]);

	//train the Gaussian Mixture Model
	void FtTrainGMM (unsigned char *pRGBSplitImg[3], unsigned char *pMask);
	void FtInitMixMean (unsigned char *pRGBSplitImg[3], unsigned char *pMask);
	void FtInitMixVar (unsigned char *pRGBSplitImg[3], unsigned char *pMask);
	void FtFillMixAcc (unsigned char *pRGBSplitImage[3], unsigned char *pMask);
	void FtFillMixConvinv (GAUSS *tmix);
	void FtZeroMixAcc ();

	//fill the look-up table with the training result
	void FtFillLUTTable();
	double FtGetProbability (int rin, int gin, int bin, GAUSS *mix);

	//track the face in the current frame
	void FtUpdateTemplate (FACE_RECT &CurFaceRect, FACE_RECT &PreFaceRect, int iStep, int flag);
	//given each pixel in current frame, calculate its likelihood value
	void FtUpdateLikelihoodMap (unsigned char *ImgData, int ImgWidth, int ImgHeight);
	//search strategy
	double FtTargetFunction(int *FaceRect, int *delta, int scale, int flag);


protected:
	int m_nImgWidth;				//the width of the image
	int m_nImgHeight;				//the height of the image
	DWORD m_nTableSize;				//the sampling rate of the color space
									//note here m_nTableSize is the byte size of the LUT table

	BOOL m_bRestrictAspectRatio;	//whether restrict the aspect ratio of the tracking result
	BOOL m_bIsInitialized;			//indicates whether the internal variables have been allocated

	unsigned char *m_pRGBSplitImg[3];	//the image stored in RGB plane format
	unsigned char *m_pMask;				//the pointer of the mask which indicates the foreground and background region

	float *m_pLUT;						//the pointer of probability look up table (LUT)
	double *m_pLikelihoodMap;			//the pointer of the likelihood map

	double *m_pom[2];
	double *m_pmo[2];
	double *m_po;

	TEMPLATE_TYPE m_TemplateType;	//the shape of the template to fit the face region

	FACE_RECT m_CurFaceRect;
	FACE_RECT m_PreFaceRect;

	double m_fTemplate[101];		//template equation

	STATE m_StateInfo[2];

};

#endif	//ifndef _FACE_TRACK_H
